;Mandelbrot Set
;The World's Crunchiest Trippiest Oldschool 8-Bit Fractal
;
;Programmed by Sean Kelly / Psych Software 2008
;BGM arranged by Sean Kelly 2008
;
;Original BGM track "Mandelbrot Set" by Jonathan Coulton
;released under CC-NC (http://creativecommons.org/licenses/by-nc/3.0/)
;and may be found at http://www.jonathancoulton.com

timerlow = $00
timerhi = $01
timerBlit = $02
nmi_count = $03

;
joypad = $04
joypad_old = $05
joypad_delta = $06

temp0 = $08
temp1 = $09
temp2 = $0A
temp3 = $0B
temp4 = $0C
temp5 = $0D
temp6 = $0E
temp7 = $0F

waitdelay = $10

; playheads into the music/action data
; init these to the locations of the arrays,
; increase them as they're used
; access arrays with indirect indexed, y=0
; e.g.
; lda #0
; tay
; lda (ch1NextIndexLo),Y
; or since nesasm sucks,
;    .db $b1,wordsNextIndexLo ; stupid... freaking... nesasm...
ch0NextIndexLo = $20
ch0NextIndexHi = $21
ch1NextIndexLo = $22
ch1NextIndexHi = $23
ch2NextIndexLo = $24
ch2NextIndexHi = $25
ch3NextIndexLo = $26
ch3NextIndexHi = $27
wordsNextIndexLo = $28
wordsNextIndexHi = $29

ch0loop = $30
ch1loop = $31
ch2loop = $32
ch3loop = $33
ch0next = $34
ch1next = $35
ch2next = $36
ch3next = $37

ch0StartHi = $40
ch1StartHi = $41
ch2StartHi = $42
ch3StartHi = $43
wordsStartHi = $44
dmcAddress1 = $45
dmcAddress2 = $46
dmcLength = $47

; MATH ROUTINE STUFF
; value format:
; flag: _____vn - o(v)erflow, (n)egative
; hi: 8-bit fixed-point 1/2 to 1/256 places
; lo: 8-bit fixed-point 1/512 to 1/64k places
; math thus works in strictly <-1,1>x<-i,i>
; and loses precision at the outer edges and near 0

; left-shift inputs
lsh16Flag = $50
lsh16Hi = $51
lsh16Lo = $52

; right-shift inputs
rsh16Flag = $53
rsh16Hi = $54
rsh16Lo = $55

; adder inputs
add16Flag1 = $56
add16Hi1 = $57
add16Lo1 = $58
add16Flag2 = $59
add16Hi2 = $5A
add16Lo2 = $5B

; multiplier inputs
mul16Flag1 = $5C
mul16Hi1 = $5D
mul16Lo1 = $5E
mul16Flag2 = $5F
mul16Hi2 = $60
mul16Lo2 = $61

; complex adder inputs
addCpxFlagR1 = $62
addCpxHiR1 = $63
addCpxLoR1 = $64
addCpxFlagI1 = $65
addCpxHiI1 = $66
addCpxLoI1 = $67
addCpxFlagR2 = $68
addCpxHiR2 = $69
addCpxLoR2 = $6A
addCpxFlagI2 = $6B
addCpxHiI2 = $6C
addCpxLoI2 = $6D

; complex squarer inputs - since Z^2+C is the core calculation,
; map the pow2 inputs to the adder 2nd inputs
; set the first adder inputs to master R/I and leave them since
; they don't get changed during the add, and then don't bother
; transferring Z between inputs for every square/add
pow2CpxFlagR = $68
pow2CpxHiR = $69
pow2CpxLoR = $6A
pow2CpxFlagI = $6B
pow2CpxHiI = $6C
pow2CpxLoI = $6D

; general state tracking
bgCol = $6E
bgRow = $6F
; center of current quarter-tile
; again, since addCpx input 1 is never mangled and we're only
; ever adding C and C===the current R/I, just use the adder
; inputs for our main position state
RFlag = $62
RHi = $63
RLo = $64
IFlag = $65
IHi = $66
ILo = $67

; center of upper-left tile
startRFlag = $70
startRHi = $71
startRLo = $72
startIFlag = $73
startIHi = $74
startILo = $75
; center of current tile
centerRFlag = $76
centerRHi = $77
centerRLo = $78
centerIFlag = $79
centerIHi = $7A
centerILo = $7B
; increment is strictly positive for X, strictly negative for Y
incrementHi = $7C
incrementLo = $7D
; quarter-increments for finding centers of each square in the tile centered at (X,Y)
quarterHi = $7E
quarterLo = $7F
; tile values
mandelTile = $80
mandelTileQ0 = $81
mandelTileQ1 = $82
mandelTileQ2 = $83

; since multiple routines use the special registers, let them back them up
mul16_xcache = $84
mandelbrot_xcache = $85
mandelbrot_ycache = $86

; variable render depth affecting speed
depth = $87
; max and countdown tracking when we need to waitNMI to minimize frameskip
itersPerNMI = $88
iterCounter = $89

; for color cycling
color0 = $8A
color1 = $8B
color2 = $8C
lastColorTime = $8D

; ZP array of palette indeces for each number of iterations
; that it takes a given point to diverge
iterColor0 = $90
iterColor1 = $91
iterColor2 = $92
iterColor3 = $93
iterColor4 = $94
iterColor5 = $95
iterColor6 = $96
iterColor7 = $97
iterColor8 = $98
iterColor9 = $99
iterColor10 = $9A
iterColor11 = $9B
iterColor12 = $9C
iterColor13 = $9D
iterColor14 = $9E
iterColor15 = $9F
; and really, the way the code is set up now,
; as much of this space as possible should be reserved for palette lookup
; I'm setting the max depth to be... $40.  So through $CF is reserved. 


irq_acache = $FA
irq_xcache = $FB
irq_ycache = $FC
nmi_acache = $FD
nmi_xcache = $FE
nmi_ycache = $FF

	.inesprg 2 ; code, footer
	.ineschr 1 ; bg, sprites
	.inesmir 1 ; standard mirror
	.inesmap 0 ; not even used, but may want to use 4 (MMC3) eventually


	.bank 0
	.org $8000	; first bank, top of the code

nmi:
	; cache regs
 sta nmi_acache
 stx nmi_xcache
 sty nmi_ycache
	; handle the interrupt and timing stuff
 lda $2002	; clear vblank interrupt
 lda #$00
 sta $2000	; reset vblank interrupt generation
 lda #%10010000	
 sta $2000	; re-set vblank interrupt generation, bg chr page
 
 inc timerBlit
 lda timerBlit
 and #%00000001
 bne nmi_no_incHi ; or in this case, no inc at all
 
 inc timerlow		; increase the timer every frame
 bne nmi_no_incHi	; if timer lobyte is 0
 inc timerhi			; then inc the hibyte
nmi_no_incHi:
 inc nmi_count
	; uncache regs
 lda nmi_acache
 ldx nmi_xcache
 ldy nmi_ycache

 rti

; theoretically only called when DMC times out, 
; but there's another source I haven't caught yet...
; this demo doesn't currently even use DMC, so :P
irq:
	; cache regs
 sta irq_acache
 stx irq_xcache
 sty irq_ycache
 ; if the interrupt came from not-DMC, return
 lda $4015
 and #$80
 beq irq_done
	; stop the sound (just in case)
 lda #$0F
 sta $4015
	; clear the interrupt
 lda #$0F ; play DMC sample then stop, highest frequency
 sta $4010
	; set up the second half of the sound
 lda #$40 ; the midline of a 6-bit PCM, lshifted 1
 sta $4011
 lda dmcAddress2
 sta $4012
 lda dmcLength
 sta $4013
 	; play the sound!
 lda #$1F
 sta $4015
 
irq_done:
	; uncache regs
 lda irq_acache
 ldx irq_xcache
 ldy irq_ycache
 rti
 


start:
 sei ; disable IRQ, reenable later with cli if needed

init:
	; set up the timer
 lda #$00
 sta timerhi
 sta timerlow
 sta timerBlit
 sta lastColorTime
 
	; set up the joypad
 sta joypad
 sta joypad_old
 sta joypad_delta
 
	; set up array pointers
 sta ch0loop
 sta ch1loop
 sta ch2loop
 sta ch3loop
 
 sta ch0NextIndexLo
 sta ch1NextIndexLo
 sta ch2NextIndexLo
 sta ch3NextIndexLo
 ; ---===BEWARE: HARDCOED ADDRESSES!===---
 lda #$A0
 sta ch0NextIndexHi
 sta ch0StartHi
 lda #$B0
 sta ch1NextIndexHi
 sta ch1StartHi
 lda #$AA
 sta ch2NextIndexHi
 sta ch2StartHi
 lda #$BA
 sta ch3NextIndexHi
 sta ch3StartHi
 
 ldy #0
 ;lda (ch0NextIndexLo),Y ; load up the current chanel 0 playhead
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 sta ch0next
 ;lda (ch1NextIndexLo),Y ; load up the current chanel 1 playhead
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 sta ch1next
 ;lda (ch2NextIndexLo),Y ; load up the current chanel 2 playhead
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 sta ch2next
 ;lda (ch3NextIndexLo),Y ; load up the current chanel 3 playhead
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 sta ch3next
 
 
 lda #0
 sta $2000 ; init PPU to name table 0, pattern tables 0, etc, w/o enabling NMI
 sta $2001 ; "turn off" PPU for massive BG load
 lda #$20
 sta $2006
 lda #$00
 sta $2006 ; set PPU address to $2000.  For once 16-bit addresses go is hibyte-first
 
 lda #0
 tax
init_bg_blank_loop1: ; write 256 0s to name table
 sta $2007
 inx
 bne init_bg_blank_loop1
init_bg_blank_loop2: ; and again- $2100-21FF
 sta $2007
 inx
 bne init_bg_blank_loop2
init_bg_blank_loop3: ; $2200-22ff
 sta $2007
 inx
 bne init_bg_blank_loop3
init_bg_blank_loop4: ; finally $2300-23ff
 sta $2007
 inx
 bne init_bg_blank_loop4

 lda #%00001010
 sta $2001 ; init PPU to color, no bg mask, sprite clip, bg on, sprites off, no color emphasis
 
	; set initial palette
 lda #$3F
 sta $2006
 lda #$00
 sta $2006
 
 lda #$0D
 sta $2007
 ;lda #$30
 lda #$01
 sta color2
 sta $2007
 ;lda #$16
 lda #$11
 sta color1
 sta $2007
 ;lda #$12
 lda #$21
 sta color0
 sta $2007
 
	; clear scrolling
 lda #0
 sta $2005
 sta $2005
 
 lda #$1F	; 000 1 1 1 1 1 - enable all channels initially
 sta $4015
 
 lda #%10010000	
 sta $2000	; set vblank interrupt generation, use 2nd chr page for bg
 
 jmp main_start


; waits a number of 1/30 second NMI-clocked frames, then returns
waitloop:
 lda waitdelay	; load the number of frames to wait
 clc
 adc timerlow	; calculate an exit time
waitloop_innerloop:
 cmp timerlow
 bne waitloop_innerloop	; loop until we reach the exit time
 rts
 
; waits a number of 1/30 second NMI-clocked frames, calling music in between
waitmusic:
 lda waitdelay	; load the number of frames to wait
 clc
 adc timerlow	; calculate an exit time
 sta waitdelay ; okay, so this mangles waitdelay
waitmusic_innerloop:
 jsr musicCheck
 lda timerlow
 cmp waitdelay
 bmi waitmusic_innerloop	; loop until we reach the exit time
 rts
 
; waits for a NMI
waitNMI:
 lda #0
 sta nmi_count
waitNMI_loop
 cmp nmi_count
 beq waitNMI_loop
 rts
 
 
; plays any notes/words whose time has come up
musicCheck:
 ldy #0 ; zero the y register
 
musicCheck_checkCh0:
 lda timerlow
 cmp ch0next
 bmi musicCheck_checkCh1 ; if timer-next<0 (we're within 128/30ths and aren't there yet), go on
 jsr processCh0
 
musicCheck_checkCh1:
 lda timerlow
 cmp ch1next
 bmi musicCheck_checkCh2
 jsr processCh1
 
musicCheck_checkCh2:
 lda timerlow
 cmp ch2next
 bmi musicCheck_checkCh3
 jsr processCh2
 
musicCheck_checkCh3:
 lda timerlow
 cmp ch3next
 bmi musicCheck_done
 jsr processCh3
 
musicCheck_done:

 rts


; increases channel 0 16-bit index
incCh0:
	; increase index
 inc ch0NextIndexLo
 bne incCh0_noIncHi
 inc ch0NextIndexHi
incCh0_noIncHi:
 rts
 
 
; increases channel 1 16-bit index
incCh1:
	; increase index
 inc ch1NextIndexLo
 bne incCh1_noIncHi
 inc ch1NextIndexHi
incCh1_noIncHi:
 rts


; increases channel 2 16-bit index
incCh2:
	; increase index
 inc ch2NextIndexLo
 bne incCh2_noIncHi
 inc ch2NextIndexHi
incCh2_noIncHi:
 rts
 

; increases channel 3 16-bit index
incCh3:
	; increase index
 inc ch3NextIndexLo
 bne incCh3_noIncHi
 inc ch3NextIndexHi
incCh3_noIncHi:
 rts


; note array format:
; [time][reg0][reg1][reg2][reg3] OR
; [time]$FF$00 - clear loop counter OR
; [time]$FF$FF - back to start
; [time]$FF[loop counter cmp val][jump dist (backwards)] OR

; processes next contents of channel 0
; assumes Y==0
processCh0:
 jsr incCh0 ; pointing at reg0 or escape on return
 ;lda (ch0NextIndexLo),Y ; load next value
     .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 cmp #$FF
 bne processCh0_normal
	; if escape value $FF is encountered, see how we loop
 jsr incCh0 ; pointing at escape data on return
 ;lda (ch0NextIndexLo),Y ; load next value
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 cmp #$00
 bne processCh0_checkReset
	; if escape-$00 is encountered, clear the loop counter
 jsr incCh0 ; pointing at next delay on return
 lda #0
 sta ch0loop
 ; here we risk a NMI incing the timer when the next entry is at the same time as this one 
 ; either code music around this or cross your fingers (or use a trigger other than beq to get to process)
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch0NextIndexLo),Y ; load next value
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 beq processCh0
 jmp processCh0_cleanup
	
processCh0_checkReset:
 cmp #$FF
 bne processCh0_doLoop
	; if escape-$FF is encountered, go to top of tune
 lda ch0StartHi
 sta ch0NextIndexHi
 lda #0
 sta ch0NextIndexLo
 jmp processCh0 ; immediately play the first note.  Hackish, but reasonable.
	
processCh0_doLoop:
 ;lda (ch0NextIndexLo),Y ; load next value
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 cmp ch0loop
 bne processCh0_doLoop_applyJump
	; else just skip to the next delay
 jsr incCh0 ; pointing at jump dist on return
 jsr incCh0 ; pointing at next delay on return
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch0NextIndexLo),Y ; load next value
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 beq processCh0
 jmp processCh0_cleanup
 
processCh0_doLoop_applyJump:
 inc ch0loop
 jsr incCh0 ; pointing at jump dist on return
 lda ch0NextIndexLo ; load the current index
 sec ; because carry is the +1 on the negation...
 ;sbc (ch0NextIndexLo),Y ; subtract the jump distance
    .db $F1,ch0NextIndexLo ; stupid... stupid... freaking... nesasm...
 ;bvc processCh0_doLoop_noPage
 ;; even if we overflowed, we want unsigned comparison, so only care
 ;; if we overflowed pos-'neg'=pos, not if 'neg'-pos='neg', e.g. if invert&add produced no carry
 ;bcs processCh0_doLoop_noPage 
 cmp ch0NextIndexLo ; try a simpler check: if new-old generates a carry (implicitly >0), roll back page
 bcc processCh0_doLoop_noPage
 dec ch0NextIndexHi ; roll back a page if needed
processCh0_doLoop_noPage:
 sta ch0NextIndexLo ; had to wait until here to be certain flags were good for branch
 ; nextIndex should now be set to the location of a valid delay
 ; it is most practical for loops to immediately play the next "note"- the delay
 ; before the jump target note may be very irrelevent to the last note, so let the
 ; delay before the $FF handle everything
 jmp processCh0
 
processCh0_normal:
	; a should already have the reg0 value
 sta $4000
 jsr incCh0 ; pointing at reg1 on return
 ;lda (ch0NextIndexLo),Y ; load next value
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 sta $4001
 jsr incCh0 ; pointing at reg2 on return
 ;lda (ch0NextIndexLo),Y ; load next value
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 sta $4002
 jsr incCh0 ; pointing at reg3 on return
 ;lda (ch0NextIndexLo),Y ; load next value
    .db $b1,ch0NextIndexLo ; stupid... freaking... nesasm...
 sta $4003
 jsr incCh0 ; pointing at next delay on return
 ; no jump, just go straight on in

processCh0_cleanup:
	; NextIndex should be pointing at the next delay now
	; use the delay to compute a new next time
 lda ch0next
 clc ; see end of processWords
 ;adc (ch0NextIndexLo),Y ; add the delay
    .db $71,ch0NextIndexLo ; stupid... freaking... nesasm...
 sta ch0next
 
 rts
 
 
; processes next contents of channel 1
; assumes Y==0
processCh1:
 jsr incCh1 ; pointing at reg0 or escape on return
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 cmp #$FF
 bne processCh1_normal
	; if escape value $FF is encountered, see how we loop
 jsr incCh1 ; pointing at escape data on return
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 cmp #$00
 bne processCh1_checkReset
	; if escape-$00 is encountered, clear the loop counter
 jsr incCh1 ; pointing at next delay on return
 lda #0
 sta ch1loop
 ; here we risk a NMI incing the timer when the next entry is at the same time as this one 
 ; either code music around this or cross your fingers (or use a trigger other than beq to get to process)
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 beq processCh1
 jmp processCh1_cleanup

processCh1_checkReset:
 cmp #$FF
 bne processCh1_doLoop
	; if escape-$FF is encountered, go to top of tune
 lda ch1StartHi
 sta ch1NextIndexHi
 lda #0
 sta ch1NextIndexLo
 jmp processCh1 ; immediately play the first note.  Hackish, but reasonable.

processCh1_doLoop:
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 cmp ch1loop
 bne processCh1_doLoop_applyJump
	; else just skip to the next delay
 jsr incCh1 ; pointing at jump dist on return
 jsr incCh1 ; pointing at next delay on return
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 beq processCh1
 jmp processCh1_cleanup
 
processCh1_doLoop_applyJump:
 inc ch1loop
 jsr incCh1 ; pointing at jump dist on return
 lda ch1NextIndexLo ; load the current index
 sec ; because carry is the +1 on the negation...
 ;sbc (ch1NextIndexLo),Y ; subtract the jump distance
    .db $F1,ch1NextIndexLo ; stupid... freaking... nesasm...
 ;bvc processCh1_doLoop_noPage
 ;; even if we overflowed, we want unsigned comparison, so only care
 ;; if we overflowed pos-'neg'=pos, not if 'neg'-pos='neg', e.g. if invert&add produced no carry
 ;bcs processCh1_doLoop_noPage 
 cmp ch1NextIndexLo ; try a simpler check: if new-old generates a carry (implicitly >0), roll back page
 bcc processCh1_doLoop_noPage
 dec ch1NextIndexHi ; roll back a page if needed

processCh1_doLoop_noPage:
 sta ch1NextIndexLo ; had to wait until here to be certain flags were good for branch
 ; nextIndex should now be set to the location of a valid delay
 ; it is most practical for loops to immediately play the next "note"- the delay
 ; before the jump target note may be very irrelevent to the last note, so let the
 ; delay before the $FF handle everything
 jmp processCh1
 
processCh1_normal:
	; a should already have the reg0 value
 sta $4004
 jsr incCh1 ; pointing at reg1 on return
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 sta $4005
 jsr incCh1 ; pointing at reg2 on return
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 sta $4006
 jsr incCh1 ; pointing at reg3 on return
 ;lda (ch1NextIndexLo),Y ; load next value
    .db $b1,ch1NextIndexLo ; stupid... freaking... nesasm...
 sta $4007
 jsr incCh1 ; pointing at next delay on return
 ; no jump, just go straight on in

processCh1_cleanup:
	; NextIndex should be pointing at the next delay now
	; use the delay to compute a new next time
 lda ch1next
 clc ; see end of processWords
 ;adc (ch1NextIndexLo),Y ; add the delay
    .db $71,ch1NextIndexLo ; stupid... freaking... nesasm...
 sta ch1next
 
 rts
 
 
; processes next contents of channel 2
; assumes Y==0
processCh2:
 jsr incCh2 ; pointing at reg0 or escape on return
 ;lda (ch2NextIndexLo),Y ; load next value
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 cmp #$FF
 bne processCh2_normal
	; if escape value $FF is encountered, see how we loop
 jsr incCh2 ; pointing at escape data on return
 ;lda (ch2NextIndexLo),Y ; load next value
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 cmp #$00
 bne processCh2_checkReset
	; if escape-$00 is encountered, clear the loop counter
 jsr incCh2 ; pointing at next delay on return
 lda #0
 sta ch2loop
 ; here we risk a NMI incing the timer when the next entry is at the same time as this one 
 ; either code music around this or cross your fingers (or use a trigger other than beq to get to process)
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch2NextIndexLo),Y ; load next value
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 beq processCh2
 jmp processCh2_cleanup
 
processCh2_checkReset:
 cmp #$FF
 bne processCh2_doLoop
	; if escape-$FF is encountered, go to top of tune
 lda ch2StartHi
 sta ch2NextIndexHi
 lda #0
 sta ch2NextIndexLo
 jmp processCh2 ; immediately play the first note.  Hackish, but reasonable.
	
processCh2_doLoop:
 ;lda (ch2NextIndexLo),Y ; load next value
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 cmp ch2loop
 bne processCh2_doLoop_applyJump
	; else just skip to the next delay
 jsr incCh2 ; pointing at jump dist on return
 jsr incCh2 ; pointing at next delay on return
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch2NextIndexLo),Y ; load next value
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 beq processCh2
 jmp processCh2_cleanup
 
processCh2_doLoop_applyJump:
 inc ch2loop
 jsr incCh2 ; pointing at jump dist on return
 lda ch2NextIndexLo ; load the current index
 sec ; because carry is the +1 on the negation...
 ;sbc (ch2NextIndexLo),Y ; subtract the jump distance
    .db $F1,ch2NextIndexLo ; stupid... freaking... nesasm...
 ;bvc processCh2_doLoop_noPage
 ;; even if we overflowed, we want unsigned comparison, so only care
 ;; if we overflowed pos-'neg'=pos, not if 'neg'-pos='neg', e.g. if invert&add produced no carry
 ;bcs processCh2_doLoop_noPage 
 cmp ch2NextIndexLo ; try a simpler check: if new-old generates a carry (implicitly >0), roll back page
 bcc processCh2_doLoop_noPage
 dec ch2NextIndexHi ; roll back a page if needed
processCh2_doLoop_noPage:
 sta ch2NextIndexLo ; had to wait until here to be certain flags were good for branch
 ; nextIndex should now be set to the location of a valid delay
 ; it is most practical for loops to immediately play the next "note"- the delay
 ; before the jump target note may be very irrelevent to the last note, so let the
 ; delay before the $FF handle everything
 jmp processCh2
 
processCh2_normal:
	; a should already have the reg0 value
 sta $4008
 jsr incCh2 ; pointing at reg1 on return
; ;lda (ch2NextIndexLo),Y ; load next value
;    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
; sta $4009
; jsr incCh2 ; pointing at reg2 on return
 ;lda (ch2NextIndexLo),Y ; load next value
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 sta $400A
 jsr incCh2 ; pointing at reg3 on return
 ;lda (ch2NextIndexLo),Y ; load next value
    .db $b1,ch2NextIndexLo ; stupid... freaking... nesasm...
 sta $400B
 jsr incCh2 ; pointing at next delay on return
 ; no jump, just go straight on in

processCh2_cleanup:
	; NextIndex should be pointing at the next delay now
	; use the delay to compute a new next time
 lda ch2next
 clc ; see end of processWords
 ;adc (ch2NextIndexLo),Y ; add the delay
    .db $71,ch2NextIndexLo ; stupid... freaking... nesasm...
 sta ch2next
 
 rts
 
 
; processes next contents of channel 3
; assumes Y==0
processCh3:
 jsr incCh3 ; pointing at reg0 or escape on return
 ;lda (ch3NextIndexLo),Y ; load next value
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 cmp #$FF
 bne processCh3_normal
	; if escape value $FF is encountered, see how we loop
 jsr incCh3 ; pointing at escape data on return
 ;lda (ch3NextIndexLo),Y ; load next value
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 cmp #$00
 bne processCh3_checkReset
	; if escape-$00 is encountered, clear the loop counter
 jsr incCh3 ; pointing at next delay on return
 lda #0
 sta ch3loop
 ; here we risk a NMI incing the timer when the next entry is at the same time as this one 
 ; either code music around this or cross your fingers (or use a trigger other than beq to get to process)
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch3NextIndexLo),Y ; load next value
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 beq processCh3
 jmp processCh3_cleanup
 
processCh3_checkReset:
 cmp #$FF
 bne processCh3_doLoop
	; if escape-$FF is encountered, go to top of tune
 lda ch3StartHi
 sta ch3NextIndexHi
 lda #0
 sta ch3NextIndexLo
 jmp processCh3 ; immediately play the first note.  Hackish, but reasonable.
	
processCh3_doLoop:
 ;lda (ch3NextIndexLo),Y ; load next value
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 cmp ch3loop
 bne processCh3_doLoop_applyJump
	; else just skip to the next delay
 jsr incCh3 ; pointing at jump dist on return
 jsr incCh3 ; pointing at next delay on return
 ; hack: if we know the next "note" is at 0 delay, just play it
 ;lda (ch3NextIndexLo),Y ; load next value
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 beq processCh3
 jmp processCh3_cleanup
 
processCh3_doLoop_applyJump:
 inc ch3loop
 jsr incCh3 ; pointing at jump dist on return
 lda ch3NextIndexLo ; load the current index
 sec ; because carry is the +1 on the negation...
 ;sbc (ch3NextIndexLo),Y ; subtract the jump distance
    .db $F1,ch3NextIndexLo ; stupid... freaking... nesasm...
 ;bvc processCh3_doLoop_noPage
 ;; even if we overflowed, we want unsigned comparison, so only care
 ;; if we overflowed pos-'neg'=pos, not if 'neg'-pos='neg', e.g. if invert&add produced no carry
 ;bcs processCh3_doLoop_noPage 
 cmp ch3NextIndexLo ; try a simpler check: if new-old generates a carry (implicitly >0), roll back page
 bcc processCh3_doLoop_noPage
 dec ch3NextIndexHi ; roll back a page if needed
processCh3_doLoop_noPage:
 sta ch3NextIndexLo ; had to wait until here to be certain flags were good for branch
 ; nextIndex should now be set to the location of a valid delay
 ; it is most practical for loops to immediately play the next "note"- the delay
 ; before the jump target note may be very irrelevent to the last note, so let the
 ; delay before the $FF handle everything
 jmp processCh3
 
processCh3_normal:
	; a should already have the reg0 value
 sta $400C
 jsr incCh3 ; pointing at reg1 on return
; ;lda (ch3NextIndexLo),Y ; load next value
;    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
; sta $400D
; jsr incCh3 ; pointing at reg2 on return
 ;lda (ch3NextIndexLo),Y ; load next value
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 sta $400E
 jsr incCh3 ; pointing at reg3 on return
 ;lda (ch3NextIndexLo),Y ; load next value
    .db $b1,ch3NextIndexLo ; stupid... freaking... nesasm...
 sta $400F
 jsr incCh3 ; pointing at next delay on return
 ; no jump, just go straight on in

processCh3_cleanup:
	; NextIndex should be pointing at the next delay now
	; use the delay to compute a new next time
 lda ch3next
 clc ; see end of processWords
 ;adc (ch3NextIndexLo),Y ; add the delay
    .db $71,ch3NextIndexLo ; stupid... freaking... nesasm...
 sta ch3next
 
 rts
 
; ---=== handle joypad input ===---
 ; joypad = ABetUDLR - (A) (B) S(e)lect S(t)art (U)p (D)own (L)eft (R)ight
joypadUpdate:
 lda joypad
 sta joypad_old
 	; poll the joypad
 ldx #$01
 stx $4016
 dex
 stx $4016
 ldy #8 ; use y as a countdown, let x be the temp for the incoming joypad data since it's 0
 ; then shift new bits in
joypad_poll_loop:
 lda $4016
 ; ror to pop the status into carry
 .db $6A
 txa
 ; rol to put the carry into the pad lowbit
 .db $2A
 tax
 dey
 bne joypad_poll_loop
 stx joypad

 txa
 eor joypad_old ; this gives us an array of changed buttons
 and joypad ; this gives us an array of what was PRESSED since last call
 sta joypad_delta
 bne joypad_eval

joypad_eval: 
 ; check A button
 and #%10000000
 beq joypad_no_A
 lda quarterHi
 sta rsh16Hi
 lda quarterLo
 sta rsh16Lo
 jsr rsh16 ; halve the quarter-increment
 lda rsh16Lo
 ora rsh16Hi
 beq joypad_no_A ; if the quarter-increment zeroes, don't store the zoomed increment back
 lda rsh16Hi
 sta quarterHi
 lda rsh16Lo
 sta quarterLo
 lda incrementHi
 sta rsh16Hi
 lda incrementLo
 sta rsh16Lo
 jsr rsh16
 lda rsh16Hi
 sta incrementHi
 lda rsh16Lo
 sta incrementLo
 ; now recenter by moving down/right 1/4 of the old screen size or 2x1/4 of the new screen
 jsr joypad_sub_move_right
 jsr joypad_sub_move_right
 jsr joypad_sub_move_down
 jsr joypad_sub_move_down
joypad_no_A:
 ; check B button
 lda joypad_delta
 and #%01000000
 beq joypad_no_B
 lda incrementHi
 cmp #%00010000
 beq joypad_no_B ; if we're at a 1/32 increment already, don't zoom out
 sta lsh16Hi
 lda incrementLo
 sta lsh16Lo
 jsr lsh16
 lda lsh16Hi
 sta incrementHi
 lda lsh16Lo
 sta incrementLo
 lda quarterHi
 sta lsh16Hi
 lda quarterLo
 sta lsh16Lo
 jsr lsh16
 lda lsh16Hi
 sta quarterHi
 lda lsh16Lo
 sta quarterLo
 ; now recenter by moving up/left 1/2 of the old screen size or 1/4 of the new screen
 jsr joypad_sub_move_left
 jsr joypad_sub_move_up
joypad_no_B:
 ; check Select
 lda joypad_delta
 and #%00100000
 beq joypad_no_Select
 ldx depth
 dex
 bne joypad_Select_depthOK
 inx
joypad_Select_depthOK:
 stx depth
joypad_no_Select:
 ; check Start
 lda joypad_delta
 and #%00010000
 beq joypad_no_Start
 ldx depth
 inx
 txa
 and #%00111111 ; max depth we have palette lookup reserved for is $3F
 bne joypad_Start_depthOK
 dex
joypad_Start_depthOK:
 stx depth
joypad_no_Start:
 ; check Up
 lda joypad_delta
 and #%00001000
 beq joypad_no_Up
 jsr joypad_sub_move_up
joypad_no_Up:
 ;check Down
 lda joypad_delta
 and #%00000100
 beq joypad_no_Down
 jsr joypad_sub_move_down
joypad_no_Down:
 ; check Left
 lda joypad_delta
 and #%00000010
 beq joypad_no_Left
 jsr joypad_sub_move_left
joypad_no_Left:
 ; check Right
 lda joypad_delta
 and #%00000001
 beq joypad_no_Right
 jsr joypad_sub_move_right
joypad_no_Right:
 lda joypad
 sta joypad_old
 rts

; joystick subroutines that may be used both in moving and zooming 
joypad_sub_move_left:
 ; increment is 1/32 of screen, we want 1/4 of screen
 lda incrementHi
 sta lsh16Hi
 lda incrementLo
 sta lsh16Lo
 jsr lsh16
 jsr lsh16
 jsr lsh16
 lda lsh16Hi
 sta add16Hi2
 lda lsh16Lo
 sta add16Lo2
 lda #%00000001
 sta add16Flag2
 lda startRFlag
 sta add16Flag1
 lda startRHi
 sta add16Hi1
 lda startRLo
 sta add16Lo1
 jsr add16
 lda add16Flag2
 and #%00000010
 beq joypad_sub_move_left_OK ; if we moved too far, set to default -63/64
 lda #%00000001
 sta startRFlag
 lda #%11111100
 sta startRHi
 lda #0
 sta startRLo
 rts
joypad_sub_move_left_OK:
 lda add16Flag2
 sta startRFlag
 lda add16Hi2
 sta startRHi
 lda add16Lo2
 sta startRLo
 rts
 
joypad_sub_move_right:
 ; increment is 1/32 of screen, we want 1/4 of screen
 lda incrementHi
 sta lsh16Hi
 lda incrementLo
 sta lsh16Lo
 jsr lsh16
 jsr lsh16
 jsr lsh16
 lda lsh16Hi
 sta add16Hi1
 lda lsh16Lo
 sta add16Lo1
 lda #%00000000
 sta add16Flag1
 lda startRFlag
 sta add16Flag2
 lda startRHi
 sta add16Hi2
 lda startRLo
 sta add16Lo2
 jsr add16
 lda add16Flag2
 and #%00000010
 beq joypad_sub_move_right_OK ; if we moved too far, set to 63/64
 lda #%00000000
 sta startRFlag
 lda #%11111100
 sta startRHi
 lda #0
 sta startRLo
 rts
joypad_sub_move_right_OK:
 lda add16Flag2
 sta startRFlag
 lda add16Hi2
 sta startRHi
 lda add16Lo2
 sta startRLo
 rts
 
joypad_sub_move_up:
 ; increment is 1/32 of screen, we want 1/4 of screen
 lda incrementHi
 sta lsh16Hi
 lda incrementLo
 sta lsh16Lo
 jsr lsh16
 jsr lsh16
 jsr lsh16
 lda lsh16Hi
 sta add16Hi1
 lda lsh16Lo
 sta add16Lo1
 lda #%00000000
 sta add16Flag1
 lda startIFlag
 sta add16Flag2
 lda startIHi
 sta add16Hi2
 lda startILo
 sta add16Lo2
 jsr add16
 lda add16Flag2
 and #%00000010
 beq joypad_sub_move_up_OK ; if we moved too far, set to default 61/64
 lda #%00000000
 sta startIFlag
 lda #%11110100
 sta startIHi
 lda #0
 sta startILo
 rts
joypad_sub_move_up_OK:
 lda add16Flag2
 sta startIFlag
 lda add16Hi2
 sta startIHi
 lda add16Lo2
 sta startILo
 rts
 
joypad_sub_move_down:
 ; increment is 1/32 of screen, we want 1/4 of screen
 lda incrementHi
 sta lsh16Hi
 lda incrementLo
 sta lsh16Lo
 jsr lsh16
 jsr lsh16
 jsr lsh16
 lda lsh16Hi
 sta add16Hi2
 lda lsh16Lo
 sta add16Lo2
 lda #%00000001
 sta add16Flag2
 lda startIFlag
 sta add16Flag1
 lda startIHi
 sta add16Hi1
 lda startILo
 sta add16Lo1
 jsr add16
 lda add16Flag2
 and #%00000010
 beq joypad_sub_move_down_OK ; if we moved too far, set to -61/64
 lda #%00000001
 sta startIFlag
 lda #%11110100
 sta startIHi
 lda #0
 sta startILo
 rts
joypad_sub_move_down_OK:
 lda add16Flag2
 sta startIFlag
 lda add16Hi2
 sta startIHi
 lda add16Lo2
 sta startILo
 rts 
 

; cycle the colors
cycleColors:
 lda timerlow
 eor lastColorTime ; get bits which have changed
 and #%11111000 ; get change in >8 frame range
 beq no_colors
 lda timerlow
 sta lastColorTime
 lda color1
 sta color2
 lda color0
 sta color1
 clc
 adc #$10
 cmp #$30
 bmi colors_finish
 clc
 adc #$D1
 cmp #$0D
 bmi colors_finish
 lda #$01
colors_finish:
 sta color0
no_colors:
 rts

; -----===== MATH CODE =====-----

; 16-bit left-shift
; done in place in lsh16*
; overflow flag set if a 1 rotates out of high byte
; worth noting that this doesn't mangle the outgoing carry
; even if overflow occurs, so it can be used kinda like
; a normal rol
; could probably be inlined to speed things up
lsh16:
 clc
 ; rol lsh16Lo
 .db $26, lsh16Lo
 ;rol lsh16Hi
 .db $26, lsh16Hi
 bcc lsh16_no_overflow
 lda lsh16Flag
 ora #%00000010
 sta lsh16Flag
lsh16_no_overflow:
 rts


; 16-bit right-shift
; done in place in rsh16*
; worth noting that this doesn't mangle the outgoing carry
; so it can be used kinda like a normal ror
; could be inlined to speed things up
rsh16:
 clc
 ; ror rsh16Hi
 .db $66, rsh16Hi
 ; ror rsh16Lo
 .db $66, rsh16Lo
 rts

; 16-bit adder
; result ends up in add16*2
; if a positive and negative value are added, the positive is
; moved into add16*1 and the negative is moved into add16*2 and 
; then subtracted
; sets overflow flag if addition produces a carry or if one
; existing overflow flag is set
add16:
 lda add16Flag1
 eor add16Flag2
 and #$00000001
 bne add16_subtract
 clc
 lda add16Lo1
 adc add16Lo2
 sta add16Lo2
 lda add16Hi1
 adc add16Hi2
 sta add16Hi2
 lda add16Flag2 ; get thie load out of the way, since we need it on either path and it doesn't kill the carry
 bcc add16_no_overflow
 ora #%00000010
add16_no_overflow:
 ora add16Flag1 ; safe, since negative flags are equal
 sta add16Flag2
 rts
; almost deserving of another routine, but conceptually I want to 
; keep it grouped with the adder and never call it independently
add16_subtract:
 lda add16Flag1
 and #%00000001
 beq add16_sub_no_switch
 lda add16Flag1
 sta temp0
 lda add16Hi1
 sta temp1
 lda add16Lo1
 sta temp2
 lda add16Flag2
 sta add16Flag1
 lda add16Hi2
 sta add16Hi1
 lda add16Lo2
 sta add16Lo1
 lda temp0
 sta add16Flag2
 lda temp1
 sta add16Hi2
 lda temp2
 sta add16Lo2
add16_sub_no_switch: ; by now the positive number is in 1 and the negative in 2
 ; we know as much about the old flags as we need to.  We can't possibly overflow 
 ; here, so assume result is positive for now...
 lda #0
 sta add16Flag2
 lda add16Lo1
 ; sanity note: the sbc instruction only does proper subtraction with the carry set.
 ; if the carry clears during the op, it indicates a borrow was required.
 ; convenient for us, since for the high byte, we want the result to be one less than
 ; normal if the low byte generated a borrow.
 ; almost like they wanted the processor to be able to do 16-bit math ;)
 sec
 sbc add16Lo2
 sta add16Lo2
 lda add16Hi1
 sbc add16Hi2
 sta add16Hi2
 ; and now it gets fun.  If the high subtraction also borrowed, we have a wrapped-around 
 ; 2s-comp negative on our hands when we want a positive with a sign bit. (I think...)
 bcs add16_sub_no_invert
 ; invert the whole 16-bit value and add "1"
 ; we already have add16Hi2 in A
 eor #%11111111 ; no NOT op, so xor with 1s
 sta add16Hi2
 lda add16Lo2
 eor #%11111111
 clc
 adc #$01
 sta add16Lo2
 lda add16Hi2
 adc #0 ; if a carry was generated on the lobyte add, add it here
 sta add16Hi2
 lda #%00000001 ; we now know this result is negative
 sta add16Flag2
add16_sub_no_invert:
 rts

; 16-bit multiplier
; result ends up in mul16*2
; if either input is overflowed, returns immediately with overflow set
; otherwise, cannot overflow since inputs strictly <1
mul16:
 stx mul16_xcache
 lda mul16Flag1
 ora mul16Flag2
 and #%00000010
 beq mul16_no_overflow
 sta mul16Flag2 ; what?  You want I should preserve the negative flag on a junk call?
mul16_no_overflow:
 ; basic shift-and-add method
 ; keep halving mul*1 and popping bits off mul*2
 ; if the bit off mul*2 is a 1, add the remaining mul*1 to result
 lda #0
 sta lsh16Flag
 sta rsh16Flag
 sta add16Flag1
 sta add16Flag2
 sta add16Hi2
 sta add16Lo2
 lda mul16Hi1
 sta rsh16Hi
 lda mul16Lo1
 sta rsh16Lo
 lda mul16Hi2
 sta lsh16Hi
 lda mul16Lo2
 sta lsh16Lo
 clc
 jsr rsh16 ; since the highest power place in mul*2 is 1/2
 ldx #0
mul16_loop:
 jsr lsh16 ; which pops the shifted-out bit into carry
 bcc mul16_loop_no_add ; so we can act on it right away
 lda rsh16Hi
 sta add16Hi1
 lda rsh16Lo
 sta add16Lo1
 jsr add16
mul16_loop_no_add:
 jsr rsh16
 inx
 cpx #15 ; after 15 rshs, we're guaranteed to have 0 in the rsh input
 bne mul16_loop
; visual break to bookend the loop
 lda add16Hi2
 sta mul16Hi2
 lda add16Lo2
 sta mul16Lo2
 lda mul16Flag1
 eor mul16Flag2
 sta mul16Flag2 ; safe, since we know we can't have overflowed, so only the sign flags might be unequal, producing a negative
 ldx mul16_xcache
 rts

; complex number adder
; no tricky stuff here
; honestly, inlining this calculation would probably be
; a big time gain, but modularity is more reader-friendly
; result is stored in addCpx*2
addCpx:
 lda addCpxFlagR1
 sta add16Flag1
 lda addCpxHiR1
 sta add16Hi1
 lda addCpxLoR1
 sta add16Lo1
 lda addCpxFlagR2
 sta add16Flag2
 lda addCpxHiR2
 sta add16Hi2
 lda addCpxLoR2
 sta add16Lo2
 jsr add16
 lda add16Flag2
 sta addCpxFlagR2
 lda add16Hi2
 sta addCpxHiR2
 lda add16Lo2
 sta addCpxLoR2
 ; visual break to distinguish R/I sections
 lda addCpxFlagI1
 sta add16Flag1
 lda addCpxHiI1
 sta add16Hi1
 lda addCpxLoI1
 sta add16Lo1
 lda addCpxFlagI2
 sta add16Flag2
 lda addCpxHiI2
 sta add16Hi2
 lda addCpxLoI2
 sta add16Lo2
 jsr add16
 lda add16Flag2
 sta addCpxFlagI2
 lda add16Hi2
 sta addCpxHiI2
 lda add16Lo2
 sta addCpxLoI2
 rts

; complex Pow2 (square) routine
; R*R-I*I + 2*R*I*i
; result goes back in inputs when done, but is
; stored in temps temporarily
; does not assign flags itself, but add/multiply/shift
; flags propagate out
pow2Cpx:
 lda pow2CpxFlagR
 sta mul16Flag1
 sta mul16Flag2
 lda pow2CpxHiR
 sta mul16Hi1
 sta mul16Hi2
 lda pow2CpxLoR
 sta mul16Lo1
 sta mul16Lo2
 jsr mul16
 lda mul16Flag2
 sta temp0
 lda mul16Hi2
 sta temp1
 lda mul16Lo2
 sta temp2
 ; visual break after squaring R: for reference temp0/1/2 have R^2
 lda pow2CpxFlagI
 sta mul16Flag1
 sta mul16Flag2
 lda pow2CpxHiI
 sta mul16Hi1
 sta mul16Hi2
 lda pow2CpxLoI
 sta mul16Lo1
 sta mul16Lo2
 jsr mul16
 lda mul16Flag2
 eor #%00000001 ; flip the sign
 ; visual break after squaring I
 sta add16Flag2 ; and dump directly into the add inputs to save on transfers
 lda mul16Hi2
 sta add16Hi2
 lda mul16Lo2
 sta add16Lo2
 lda temp0
 sta add16Flag1
 lda temp1
 sta add16Hi1
 lda temp2
 sta add16Lo1
 jsr add16
 lda add16Flag2
 sta temp0
 lda add16Hi2
 sta temp1
 lda add16Lo2
 sta temp2
 ; visual break after subtracting I^2 from R^2: temp0/1/2 now have R^2 - I^2
 lda pow2CpxFlagR
 sta mul16Flag1
 lda pow2CpxHiR
 sta mul16Hi1
 lda pow2CpxLoR
 sta mul16Lo1
 lda pow2CpxFlagI
 sta mul16Flag2
 lda pow2CpxHiI
 sta mul16Hi2
 lda pow2CpxLoI
 sta mul16Lo2
 jsr mul16
 lda mul16Flag2
 sta lsh16Flag
 lda mul16Hi2
 sta lsh16Hi
 lda mul16Lo2
 sta lsh16Lo
 ; visual break after computing R*I*2
 jsr lsh16
 lda lsh16Flag
 sta pow2CpxFlagI
 lda lsh16Hi
 sta pow2CpxHiI
 lda lsh16Lo
 sta pow2CpxLoI
 lda temp0
 sta pow2CpxFlagR
 lda temp1
 sta pow2CpxHiR
 lda temp2
 sta pow2CpxLoR
 rts
 
; mandelbrot iterator
; the only purpose of this routine
; is to return in mandelTile a 2-bit color
; for the number of iterations required to
; diverge at the given C (R*+I*)
mandelbrot:
 stx mandelbrot_xcache
 sty mandelbrot_ycache
 ; Let Z1 be 0^2+C
 lda RFlag
 sta pow2CpxFlagR
 lda RHi
 sta pow2CpxHiR
 lda RLo
 sta pow2CpxLoR
 lda IFlag
 sta pow2CpxFlagI
 lda IHi
 sta pow2CpxHiI
 lda ILo
 sta pow2CpxLoI
 ldx #$00
 ldy iterCounter
mandelbrot_loop: ; iterate up to 12 times (x==11) or until Z overflows
 ; Z^2
 jsr pow2Cpx
 ; +C, since the pow2 in/out is set to the same address as the adder input 2
 ; and the (non-mangled) adder input 1 is the same address as the master R/I 
 jsr addCpx 
 
 dey
 bne mandelbrot_loop_no_NMI
 jsr waitNMI
 jsr musicCheck
 ldy itersPerNMI
 
mandelbrot_loop_no_NMI:
 
 lda addCpxFlagR2
 and #%00000010
 bne mandelbrot_break
 lda addCpxFlagI2
 and #%00000010
 bne mandelbrot_break
 cpx depth ; usually 5, 11, or 15
 beq mandelbrot_break
 inx
 jmp mandelbrot_loop
mandelbrot_break:
 ;lda iterColor0,x ; call me paranoid, but if nesasm mangles some Y-offset codes, I don't trust it for X-offsets
 .db $B5,iterColor0
 sta mandelTile
 sty iterCounter
 ldx mandelbrot_xcache
 ldy mandelbrot_ycache
 rts

  
; -----===== MAIN CODE =====-----

main_start:
 ; variable render depth affecting speed
 ; expect tiles/frame a fraction lower than 
 ; iters/quarter * quarters/tile * frames/iter
 ; = depth * 4 / itersPerNMI
 lda #$09
 sta depth
 
 ; set up a countdown for number of Z^2+Cs we can 
 ; do each frame without frameskipping
 lda #$08
 sta itersPerNMI
 sta iterCounter
 
 ; start screen range is -1 to 1 on real and -i to i sans 1 tile row at top and bottom on imaginary
 ; screen is 32 tiles by 30 tiles, so our step is 2/32nd or 1/16 or .00010000 00000000
 ; our start coordinate is tile center at -31/32-1/64 or -63/64 = -.11111100 00000000 on real
 ; 30/32+1/64 or 61/64 = .11110100 00000000 on imaginary
 
 ; start r at -63/64 (normal)
 lda #%00000001
 sta startRFlag
 lda #%11111100
 sta startRHi
 lda #%00000000
 sta startRLo
 ; start r at -255/256 (4x zoom lower-left of upper-left)
 ;lda #%00000001
 ;sta startRFlag
 ;lda #%11111111
 ;sta startRHi
 ;lda #%00000000
 ;sta startRLo
 ; start r at 1/64 (2x zoom upper-right)
 ;lda #%00000000
 ;sta startRFlag
 ;lda #%00000100
 ;sta startRHi
 ;lda #%00000000
 ;sta startRLo
 
 ; start i at 61/64 (normal)
 lda #%00000000
 sta startIFlag
 lda #%11110100
 sta startIHi
 lda #%00000000
 sta startILo
 
 ; start i at 127/256 (4x zoom ll of ul)
 ;lda #%00000000
 ;sta startIFlag
 ;lda #%01111111
 ;sta startIHi
 ;lda #%00000000
 ;sta startILo
 
 lda #%00000000
 sta incrementLo
 sta rsh16Lo 
 ; 1/16 (normal)
 lda #%00010000
 ; 1/32 (2x)
 ;lda #%00001000
 ; 1/64 (4x)
 ;lda #%00000100
 sta incrementHi
 sta rsh16Hi
 jsr rsh16
 jsr rsh16
 lda rsh16Hi
 sta quarterHi
 lda rsh16Lo
 sta quarterLo
 
main_render_start:
 ; tile value lookup array - this takes each possible number 
 ; of iterations until divergence and assigns it a palette color
 lda #0 ; palette value
 ldx #0 ; count-up
main_render_palette_loop:
 sec
 adc #0
 and #%00000011
 cmp #0
 bne main_render_palette_loop_noskip
 lda #%00000001
main_render_palette_loop_noskip:
 sta iterColor0,x
 inx
 cpx depth
 bne main_render_palette_loop
 lda #0 ; palette value
 sta iterColor0,x


 lda #0
 sta bgCol
 sta bgRow
 
 lda startRFlag
 sta centerRFlag
 lda startRHi
 sta centerRHi
 lda startRLo
 sta centerRLo
 lda startIFlag
 sta centerIFlag
 lda startIHi
 sta centerIHi
 lda startILo
 sta centerILo
 
main_render_loop:
 jsr musicCheck

 ; ---=== quarter-tile 0, above-left of center ===---
 lda centerRFlag
 sta add16Flag1
 lda centerRHi
 sta add16Hi1
 lda centerRLo
 sta add16Lo1
 
 lda #%00000001
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta RFlag
 lda add16Hi2
 sta RHi
 lda add16Lo2
 sta RLo
 
 lda centerIFlag
 sta add16Flag1
 lda centerIHi
 sta add16Hi1
 lda centerILo
 sta add16Lo1
 
 lda #%00000000
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta IFlag
 lda add16Hi2
 sta IHi
 lda add16Lo2
 sta ILo
 
 jsr mandelbrot
 
 lda mandelTile
 sta mandelTileQ0
 
 ; ---=== quarter-tile 1, above-right of center ===---
main_render_loop_q1:
 lda centerRFlag
 sta add16Flag1
 lda centerRHi
 sta add16Hi1
 lda centerRLo
 sta add16Lo1
 
 lda #%00000000
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta RFlag
 lda add16Hi2
 sta RHi
 lda add16Lo2
 sta RLo
 
 lda centerIFlag
 sta add16Flag1
 lda centerIHi
 sta add16Hi1
 lda centerILo
 sta add16Lo1
 
 lda #%00000000
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta IFlag
 lda add16Hi2
 sta IHi
 lda add16Lo2
 sta ILo
 
 jsr mandelbrot
 
 lda mandelTile
 clc
 ; rol rol
 .db $2A, $2A
 sta mandelTileQ1
 
 ; ---=== quarter-tile 2, below-left of center ===---
main_render_loop_q2:
 lda centerRFlag
 sta add16Flag1
 lda centerRHi
 sta add16Hi1
 lda centerRLo
 sta add16Lo1
 
 lda #%00000001
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta RFlag
 lda add16Hi2
 sta RHi
 lda add16Lo2
 sta RLo
 
 lda centerIFlag
 sta add16Flag1
 lda centerIHi
 sta add16Hi1
 lda centerILo
 sta add16Lo1
 
 lda #%00000001
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta IFlag
 lda add16Hi2
 sta IHi
 lda add16Lo2
 sta ILo
 
 jsr mandelbrot
 
 lda mandelTile
 clc
 ; rol rol rol rol
 .db $2A, $2A, $2A, $2A
 sta mandelTileQ2
 
 ; ---=== quarter-tile 3, below-right of center ===---
main_render_loop_q3:
 lda centerRFlag
 sta add16Flag1
 lda centerRHi
 sta add16Hi1
 lda centerRLo
 sta add16Lo1
 
 lda #%00000000
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta RFlag
 lda add16Hi2
 sta RHi
 lda add16Lo2
 sta RLo
 
 lda centerIFlag
 sta add16Flag1
 lda centerIHi
 sta add16Hi1
 lda centerILo
 sta add16Lo1
 
 lda #%00000001
 sta add16Flag2
 lda quarterHi
 sta add16Hi2
 lda quarterLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta IFlag
 lda add16Hi2
 sta IHi
 lda add16Lo2
 sta ILo
 
 jsr mandelbrot
 
 lda mandelTile
 clc
 ; rol rol rol rol rol rol
 .db $2A, $2A, $2A, $2A, $2A, $2A
 ora mandelTileQ2
 ora mandelTileQ1
 ora mandelTileQ0
 
 sta mandelTile ; final composited tile value
 
 jsr cycleColors
 
 jsr waitNMI
 
 lda #0
 sta $2001 ; "turn off" PPU for BG edit
 
 ; update the palette while we have time
 lda #$3F
 sta $2006
 lda #$00
 sta $2006
 
 lda #$0F
 sta $2007
 lda color2
 sta $2007
 lda color1
 sta $2007
 lda color0
 sta $2007
 
 ; ---=== ripped code from Still Alive demo to convert row/col to ppu address
 lda bgRow
 and #%00011000 ; get the "number of 8s" in the Y pos
 clc
 ;ror
 ;ror
 ;ror
    .db $6a,$6a,$6a ; 3 rors in raw
 ; no need to clc- it was cleared, and no way we could have shifted a 1 in
 adc #$20 ; add it to the nametable highbyte
 sta $2006
 lda bgRow
 and #%00000111 ; get the # rows within the quarter
 clc
 ;rol
 ;rol
 ;rol
 ;rol
 ;rol ; shift to the high end of the accumulator
    .db $2a,$2a,$2a,$2a,$2a ; 5 rols in raw
 ; again, by definition, no clc needed
 adc bgCol ; add the final offset from xpos
 sta $2006
 
 ; place our chosen tile, and a blank so we can see where we are
 lda mandelTile
 sta $2007
 lda #0
 sta $2007
 
 lda #%00001010
 sta $2001 ; return PPU to color, no bg mask, sprite clip, bg on, sprites off, no color emphasis

	; clear scrolling
 lda #0
 sta $2005
 sta $2005
 
 ; before we compute a position change, see that we aren't just resetting
 jsr joypadUpdate
 lda joypad_delta
 beq main_render_loop_newcol
 jmp main_render_start
 
main_render_loop_newcol:
 ; increment all our x-axis stuff unless we're just going to increment our y stuff and reset it
 inc bgCol
 lda bgCol
 cmp #$20
 beq main_render_loop_newrow
 
 lda centerRFlag
 sta add16Flag1
 lda centerRHi
 sta add16Hi1
 lda centerRLo
 sta add16Lo1
 
 lda #%00000000
 sta add16Flag2
 lda incrementHi
 sta add16Hi2
 lda incrementLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta centerRFlag
 lda add16Hi2
 sta centerRHi
 lda add16Lo2
 sta centerRLo
 ; if we successfully incremented our x position, we're still going
 jmp main_render_loop

main_render_loop_newrow:
 ; increment all our y-axis stuff unless we find we're already offscreen
 inc bgRow
 lda bgRow
 cmp #$1E
 beq end_loop
 
 ; reset our col
 lda #0
 sta bgCol
 ; reset our real value
 lda startRFlag
 sta centerRFlag
 lda startRHi
 sta centerRHi
 lda startRLo
 sta centerRLo
 
 ; and advance the y-axis stuff
 lda centerIFlag
 sta add16Flag1
 lda centerIHi
 sta add16Hi1
 lda centerILo
 sta add16Lo1
 
 lda #%00000001
 sta add16Flag2
 lda incrementHi
 sta add16Hi2
 lda incrementLo
 sta add16Lo2
 
 jsr add16 ; which may mangle both inputs if it has to switch, so we can't re-use either
 
 lda add16Flag2
 sta centerIFlag
 lda add16Hi2
 sta centerIHi
 lda add16Lo2
 sta centerILo
 ; if we successfully incremented our y position, we're still going
 jmp main_render_loop
 
 
 
 
end_loop:
 jsr cycleColors
 jsr waitNMI
 
 lda #0
 sta $2001 ; "turn off" PPU for BG edit
 
 ; update the palette
 lda #$3F
 sta $2006
 lda #$00
 sta $2006
 
 lda #$0F
 sta $2007
 lda color2
 sta $2007
 lda color1
 sta $2007
 lda color0
 sta $2007
 
 ;; apparently if we leave the PPU address in palette memory, it flatscreens
 ; that, or I'm overrunning vblank and Nestopia is hating me...
 lda #$20
 sta $2006
 lda #$00
 sta $2006
 
 lda #%00001010
 sta $2001 ; return PPU to color, no bg mask, sprite clip, bg on, sprites off, no color emphasis

	; clear scrolling
 lda #0
 sta $2005
 sta $2005
 
 jsr musicCheck
 jsr joypadUpdate
 lda joypad_delta
 bne end_loop_reset
 clc
 bcc end_loop
end_loop_reset:
 jmp main_render_start


 jmp init	; loop tune - should never get here...

; store this where the PC should never reach 
;						----====DEBUG TRAP====----
 lda #$3F
 sta $2006
 lda #$00
 sta $2006
 
 lda #$31
 sta $2007
 
debug: ; trap the code here
 clc
 bcc debug
;						----====/DEBUG====----
 
	
; a reference from nesmus
;unsigned int pitcharray[]=
;	{
;	//A 55
;	2033,1928,1803,1720,1620,1531,1433,1363,1285,1215,1140,1075,
;	//A 110
;	1016,955,908,853,804,760,716,677,638,604,570,537,
;	//A 220
;	507,479,452,426,403,379,359,339,320,301,284,269,
;	//A 440
;	253,239,225,213,201,190,179,169,159,150,142,134,
;	//A 880
;	126,119,112,106,100,94,89,84,79,75,70,66,
;	//A 1760
;	63,59,56,52,49,47,44,41,39,37,35,33
;	};

;	{
;	// A1
;	$7F1,$788,$70B
;	// C2
;	$6B8,$654,$5FB,1433,$553, $505,$4BF,$474,$433,$3F8,$3BB,$38C,
;	// C3
;	$355,$324,$2F8,$2CC,$2A5, $27E,$25C,$23A,$219,$1FB,$1DF,$1C4,
;	// C4
;	$1AA,$193,$17B,$167,$153, $140,$12D,$11C,$10D,$0FD,$0EF,$0E1,
;	// C5
;	$0D5,$0C9,$0BE,$0B3,$0A9, $09F,$096,$08E,$086,$07E,$077,$070,
;	// C6
;	$06A,$064,$05E,89,$054, $04F,$04B,$046,66,$03F,$03B,$038,
;	// C7
;	$034,49,47,44,41, 39,37,35,33
;	};


; for reference
; cc l e vvvv - duty [c]ycle, [l]ength ctr disable (loop), [e]nvelope disable, [v]olume
; w uuu d sss - s[w]eep enable, [u]pdate rate, [d]irection (1=down), waveform right-[s]hift
; wwwwwwww - [w]avelength lsb
; lllll www - [l]ength ctr load, [w]avelength msb
    .bank 1
;				----==== CHANNEL 0 ====----
	.org $A000
ch0Data:
	; intro loop
    .db $00
	.db $FF,$00, $00
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$2D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$FD,$40, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$53,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db $FF,3,83, $00
	; verse 1
	.db %10000111,$00,$E1,$40, $0C ; path-
	.db %10000111,$00,$E1,$40, $06 ; o-
	.db %10000111,$00,$E1,$40, $0C ; log-
	.db %10000111,$00,$FD,$40, $0C ; ical
	.db %10000111,$00,$FD,$40, $0C ; mon-
	.db %10000111,$00,$0D,$41, $06 ; sters
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; cried
	.db %10000111,$00,$0D,$41, $06 ; the
	.db %10000111,$00,$FD,$40, $0C ; terri-
	.db %10000111,$00,$FD,$40, $0C ; fied
	.db %10000111,$00,$FD,$40, $06 ; math-
	.db %10000111,$00,$0D,$41, $06 ; i-
	.db %01000011,$00,$C4,$41, $06 ; (ma-)
	.db %10000111,$00,$0D,$41, $0C ; ti-
	.db %10000111,$00,$2D,$41, $06 ; tian
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$2D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; ev-
	.db %10000111,$00,$0D,$41, $06 ; ery
	.db %10000111,$00,$FD,$40, $06 ; one
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$FD,$40, $06 ; of
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$FD,$40, $06 ; them
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$0D,$41, $06 ; a
	.db %10000111,$00,$0D,$41, $0C ; splin-
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; ter
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; in
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$53,$41, $06 ; my
	.db %10000111,$00,$0D,$41, $06 ; eye
	
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$2D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$FD,$40, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$53,$41, $06
	.db %01000011,$00,$C4,$41, $06
	
	; verse 1 iter 2
	.db %10000111,$00,$E1,$40, $0C ; pea-
	.db %10000111,$00,$E1,$40, $06 ; no
	.db %10000111,$00,$E1,$40, $0C ; space
	.db %10000111,$00,$FD,$40, $0C ; and the
	.db %10000111,$00,$FD,$40, $0C ; koch
	.db %10000111,$00,$0D,$41, $06 ; curve
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; I fear
	.db %10000111,$00,$0D,$41, $06 ; the
	.db %10000111,$00,$FD,$40, $0C ; can-
	.db %10000111,$00,$FD,$40, $0C ; tor
	.db %10000111,$00,$FD,$40, $06 ; tern-
	.db %10000111,$00,$0D,$41, $06 ; a-
	.db %01000011,$00,$C4,$41, $06 ; (-ry)
	.db %10000111,$00,$0D,$41, $0C ; -
	.db %10000111,$00,$2D,$41, $06 ; set
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$2D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; the
	.db %10000111,$00,$0D,$41, $06 ; sier-
	.db %10000111,$00,$FD,$40, $06 ; pin-
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$FD,$40, $06 ; sky
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$FD,$40, $06 ; gas-
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$0D,$41, $06 ; ked
	.db %10000111,$00,$0D,$41, $0C ; makes
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; me
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$2D,$41, $06 ; want
	.db %01000011,$00,$C4,$41, $06
	.db %10000111,$00,$53,$41, $06 ; to
	.db %10000111,$00,$0D,$41, $06 ; cry
	
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$2D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$FD,$40, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$0D,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %01000011,$00,$C4,$41, $06
	.db %10011100,$00,$53,$41, $06 ; then
	.db %10011100,$00,$C4,$41, $06 ; a
	
	; refrain 1
	.db %10011100,$00,$93,$41, $06 ; mil-
	.db %10011100,$00,$C9,$40, $0C ; lion
	.db %10011100,$00,$E1,$40, $12 ; miles
	.db %10011100,$00,$C4,$41, $06 ; a
	.db %10011100,$00,$E1,$40, $1E ; way
	.db %10010111,$00,$0D,$41, $12 
	.db %10011100,$00,$93,$41, $06 ; a
	.db %10011100,$00,$C9,$40, $06 ; but-
	.db %10011100,$00,$B3,$40, $06 ; ter-
	.db %10011100,$00,$A9,$40, $1E ; fly
	.db %10011100,$00,$B3,$40, $0C ; flapped
	.db %10011100,$00,$E1,$40, $0C ; its
	.db %10011100,$00,$0D,$41, $12 ; wings
	.db %10011100,$00,$53,$41, $06 ; on
	.db %10011100,$00,$67,$41, $06 ; a
	.db %10011100,$00,$93,$41, $06 ; cold
	.db %10011100,$00,$C9,$40, $0C ; nov-
	.db %10011100,$00,$E1,$40, $12 ; em-
	.db %10011100,$00,$C4,$41, $06 ; ber
	.db %10011100,$00,$E1,$40, $1E ; day
	.db %10010111,$00,$0D,$41, $0C
	.db %10011100,$00,$2D,$41, $06 ; and
	.db %10011100,$00,$0D,$41, $06 ; a
	.db %10011100,$00,$FD,$10, $0C ; man
	.db %10011100,$00,$FD,$00, $06 ; named
	.db %10011100,$00,$FD,$40, $0C ; ben-
	.db %10011100,$00,$0D,$11, $0C ; oit
	.db %10011100,$00,$0D,$41, $0C ; man-
	.db %10011100,$00,$2D,$11, $0C ; del-
	.db %10011100,$00,$2D,$41, $0C ; brot
	.db %10011100,$00,$53,$41, $12 ; was
	
	; chorus - yeah, this skips an iteration, so sue me, my space is limited.
	.db %10010111,$00,$0D,$79, $0C 
	.db %10010111,$00,$0D,$79, $0C 
	.db %10010111,$00,$2D,$79, $0C
	.db %10010111,$00,$2D,$79, $0C
	.db %10010111,$00,$0D,$79, $0C 
	.db %10010111,$00,$0D,$79, $0C 
	.db %10010111,$00,$2D,$79, $0C
	.db %10000100,$00,$53,$41, $06 ; take 
	.db %10000100,$00,$2D,$41, $06 ; a
	.db $FF,$00, $00
	.db %10000100,$00,$0D,$41, $0C ; point / com-
	.db %10000100,$00,$0D,$41, $0C ; called / plex
	.db %10000100,$00,$2D,$41, $0C ; Z / plane
	.db %10000100,$00,$53,$41, $06 ; in / and
	.db %10000100,$00,$2D,$41, $06 ; the /let
	.db $FF,1,28, $00
	.db $FF,$00, $00
	.db %10000100,$00,$0D,$41, $06 ; Z / 2
	.db %10000100,$00,$0D,$41, $06 ; 1 / be
	.db %10000100,$00,$0D,$41, $06 ; be / Z
	.db %10000100,$00,$0D,$41, $06 ; Z / 1
	.db %10000100,$00,$FD,$40, $0C ; squared
	.db %10000100,$00,$FD,$40, $06 ; plus
	.db %10000101,$00,$E1,$40, $06 ; C
	.db %00010101,$00,$0D,$41, $18 
	.db %00010101,$00,$2D,$41, $0C
	.db %10000100,$00,$53,$41, $06 ; let
	.db %10000100,$00,$2D,$41, $06 ; Z
	.db $FF,1,63, $00
	.db %10000100,$00,$0D,$41, $06 ; 3
	.db %10000100,$00,$0D,$41, $06 ; be
	.db %10000100,$00,$0D,$41, $06 ; Z
	.db %10000100,$00,$0D,$41, $06 ; 2
	.db %10000100,$00,$FD,$40, $06 ; squa-
	.db %10000100,$00,$0D,$41, $06 ; red
	.db %10000100,$00,$FD,$40, $06 ; plus
	.db %10000101,$00,$E1,$40, $06 ; C
	.db %10010101,$00,$C4,$41, $0C 
	.db %10000111,$00,$0D,$41, $0C ; and
	.db %10000111,$00,$2D,$41, $06 ; so
	.db %10000111,$00,$53,$41, $06 ; on
	.db %10000111,$00,$2D,$41, $06 ; if
	.db %10000111,$00,$0D,$41, $06 ; the
	.db %10000111,$00,$FD,$40, $0C ; series
	.db %10000111,$00,$0D,$41, $0C ; of
	.db %10000111,$00,$2D,$41, $0C ; Zs
	.db %10000111,$00,$53,$41, $06 ; will
	.db %10000111,$00,$2D,$41, $0C ; al-
	.db %10000111,$00,$0D,$41, $0C ; ways
	.db %10000111,$00,$FD,$40, $0C ; stay
	.db %10010101,$00,$C4,$41, $06
	.db %10010101,$00,$2D,$41, $06
	.db %10010101,$00,$FD,$40, $06
	.db %10000111,$00,$0D,$41, $0C ; close
	.db %10000111,$00,$2D,$41, $06 ; to
	.db %10000111,$00,$2D,$41, $0C ; C
	.db %10000111,$00,$2D,$41, $06 ; and
	.db %10000111,$00,$2D,$41, $06 ; nev-
	.db %10000111,$00,$53,$41, $06 ; er
	.db %10000111,$00,$0D,$41, $0C ; trend
	.db %10000111,$00,$2D,$41, $06 ; a-
	.db %10000111,$00,$2D,$41, $0C ; way
	.db %10010101,$00,$C4,$41, $06
	.db %10000111,$00,$53,$41, $06 ; then
	.db %10000111,$00,$67,$41, $06 ; that
	.db %10000111,$00,$53,$41, $06 ; point
	.db %10000111,$00,$53,$41, $06 ; is
	.db %10000111,$00,$53,$41, $06 ; in
	.db %10000111,$00,$0D,$41, $06 ; the
	.db %10000111,$00,$2D,$41, $06 ; man-
	.db %10000111,$00,$53,$41, $06 ; del-
	.db %10000111,$00,$67,$41, $0C ; brot
	.db %10000111,$00,$53,$41, $0C ; set
	.db %10010101,$00,$C4,$41, $06
	.db %10000111,$00,$0D,$41, $0C ; man-
	.db %10000111,$00,$0D,$41, $06 ; del
	.db %10000111,$00,$FD,$40, $0C ; brot
	
	.db %10000111,$00,$2D,$41, $0C ; set
	.db %10000111,$00,$2D,$41, $06 ; you're
	.db %10000111,$00,$2D,$41, $06 ; a
	.db %10000111,$00,$2D,$41, $06 ; roer-
	.db %10000111,$00,$53,$41, $0C ; schach
	.db %10000111,$00,$67,$41, $0C ; test
	.db %10000111,$00,$53,$41, $0C ; on
	.db %10000111,$00,$2D,$41, $06 ; fire
	.db %10010111,$00,$C4,$79, $06 
	.db %10010111,$00,$C4,$79, $06
	.db %10000111,$00,$53,$41, $06 ; you're
	.db %10000111,$00,$2D,$41, $06 ; a
	.db %10000111,$00,$0D,$41, $0C ; day-
	.db %10000111,$00,$0D,$41, $0C ; glow
	.db %10000111,$00,$0D,$41, $0C ; pter-
	.db %10000111,$00,$2D,$41, $06 ; a-
	.db %10000111,$00,$2D,$41, $0C ; dac
	.db %10000111,$00,$53,$41, $06 ; tyl
	.db %10010111,$00,$C4,$79, $06 
	.db %10010111,$00,$53,$79, $06
	.db %10010111,$00,$C4,$79, $06 
	.db %10010111,$00,$53,$79, $06
	.db %10000111,$00,$2D,$41, $06 ; you're
	.db %10000111,$00,$0D,$41, $06 ; a
	.db %10000111,$00,$FD,$40, $0C ; heart-
	.db %10000111,$00,$FD,$40, $0C ; shaped
	.db %10000111,$00,$FD,$40, $0C ; box
	.db %10000111,$00,$0D,$41, $06 ; of
	.db %10000111,$00,$2D,$41, $0C ; springs
	.db %10000111,$00,$0D,$41, $0C ; and
	.db %10000111,$00,$FD,$40, $0C ; wire
	.db %10000111,$00,$0D,$41, $06 ; you're
	.db %10000111,$00,$FD,$40, $0C ; one
	.db %10000111,$00,$E1,$40, $0C ; bad-
	.db %10000111,$00,$E1,$40, $0C ; ass
	.db %10000111,$00,$E1,$40, $06 ; fuck-
	.db %10000111,$00,$FD,$40, $0C ; ing
	.db %10000111,$00,$FD,$40, $0C ; frac-
	.db %10000111,$00,$0D,$41, $06 ; tal
	.db %10010111,$00,$53,$79, $06
	.db %10010111,$00,$C4,$79, $06
	.db %10010111,$00,$53,$79, $06
	.db %10010111,$00,$C4,$79, $06
	
	.db %11010111,$00,$53,$41, $06 ; and
	.db %11010111,$00,$2D,$41, $06 ; you're
	.db $FF,$00, $00
	.db %11011010,$00,$0D,$41, $06 ; just / sweep-
	.db %11011010,$00,$53,$B9, $0C ; in * / ing *
	.db %11011010,$00,$53,$41, $0C ; time / all
	.db %11011010,$00,$93,$41, $0C ; to / our
	.db %11011010,$00,$53,$41, $0C ; save / fears
	.db %11011010,$00,$2D,$41, $06 ; the / a-
	.db %11011010,$00,$0D,$41, $18 ; day / way
	.db %11010101,$00,$53,$41, $06 ; (and)
	.db %11010101,$00,$2D,$41, $06 ; you're / you'll
	.db $FF,1,48, $00
	.db %11011010,$00,$0D,$41, $0C ; change
	.db %11011010,$00,$53,$89, $06 ; the **
	.db %11011010,$00,$53,$B9, $12 ; world *
	.db %11011010,$00,$53,$41, $06 ; in
	.db %11011010,$00,$93,$41, $06 ; a
	.db %11011010,$00,$53,$41, $06 ; ti-
	.db %11011010,$00,$2D,$41, $06 ; ny
	.db %11011010,$00,$0D,$41, $24 ; way
	.db $FF,$FF ; reset
	
;				----==== CHANNEL 2 ====----	
	.org $AA00
ch2Data:
    .db $00
	; intro
	.db $FF,$00, $00
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	
	.db %00101000,$A5,$F2, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db $FF,3,67, $00
	
	; verse
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	
	.db %00101000,$A5,$F2, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$FB,$F1, $06
	
	.db %00101000,$F8,$F3, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$19,$F2, $06
	.db %00101000,$5C,$F2, $06
	
	.db %00101000,$BF,$F4, $06
	.db %00101000,$BF,$F4, $06
	.db %00101000,$5C,$F2, $06
	.db %00101000,$BF,$F4, $06
	.db %00101000,$BF,$F4, $06
	.db %00101000,$5C,$F2, $06
	.db %00101000,$19,$F2, $06
	.db %00101000,$FB,$F1, $06
	
	.db %00101000,$F8,$F3, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$8C,$F3, $06
	
	.db %00101000,$C4,$F1, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$A5,$F2, $06
	
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	
	.db %00101000,$A5,$F2, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	
	; verse 2nd iter
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	
	.db %00101000,$A5,$F2, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$FB,$F1, $06
	
	.db %00101000,$F8,$F3, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$19,$F2, $06
	.db %00101000,$5C,$F2, $06
	
	.db %00101000,$BF,$F4, $06
	.db %00101000,$BF,$F4, $06
	.db %00101000,$5C,$F2, $06
	.db %00101000,$BF,$F4, $06
	.db %00101000,$BF,$F4, $06
	.db %00101000,$5C,$F2, $06
	.db %00101000,$19,$F2, $06
	.db %00101000,$FB,$F1, $06
	
	.db %00101000,$F8,$F3, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$FB,$F1, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$8C,$F3, $06
	
	.db %00101000,$C4,$F1, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$A5,$F2, $06
	
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	
	.db %00101000,$A5,$F2, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$F8,$F3, $06
	
	; refrain
	.db %10000001,$A5,$F2, $12
	.db %10000001,$F8,$F3, $0C
	.db %10000001,$A5,$F2, $06
	.db %10000001,$F8,$F3, $0C
	.db %10000001,$A5,$F2, $0C
	.db %10000001,$8C,$F3, $06
	.db %10000001,$A5,$F2, $1E
	
	.db $FF,$00, $00
	.db %10000001,$F8,$F3, $0C
	.db %10000001,$A5,$F2, $06
	.db %10000001,$F8,$F3, $1E
	.db %10000001,$A5,$F2, $0C
	.db %10000001,$8C,$F3, $06
	.db %10000001,$A5,$F2, $1E
	.db $FF,1,27, $00
	
	.db %10000001,$BF,$F4, $0C
	.db %10000001,$24,$F3, $06
	.db %10000001,$BF,$F4, $1E
	.db %10000001,$8C,$F3, $18
	.db %00101000,$BF,$F4, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$BF,$F4, $06
	.db %00101000,$8C,$F3, $06
	
	; chorus
	.db %10000001,$A5,$F2, $30
	.db %10000001,$53,$F1, $30
	
	.db $FF,$00, $00
	.db %10000001,$53,$F1, $18
	.db %10000001,$C4,$F1, $18
	.db $FF,6,11, $00
	
	.db %10000001,$53,$F1, $0C
	.db %10000001,$A5,$F2, $18
	.db %10000001,$53,$F5, $0C
	
	.db %10000001,$F8,$F3, $0C
	.db %10000001,$A5,$F2, $06
	.db %10000001,$F8,$F3, $0C
	.db %10000001,$A5,$F2, $06
	.db %10000001,$F8,$F3, $06
	.db %10000001,$A5,$F2, $06
	
	.db %10000001,$8C,$F3, $0C
	.db %10000001,$5C,$F2, $06
	.db %10000001,$8C,$F3, $0C
	.db %10000001,$5C,$F2, $06
	.db %10000001,$8C,$F3, $06
	.db %10000001,$5C,$F2, $06
	
	.db %10000001,$33,$F4, $0C
	.db %10000001,$CC,$F2, $06
	.db %10000001,$33,$F4, $0C
	.db %10000001,$CC,$F2, $06
	.db %10000001,$33,$F4, $06
	.db %10000001,$CC,$F2, $06
	
	.db %00101000,$24,$F3, $06
	.db %00101000,$24,$F3, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$CC,$F2, $06
	.db %00101000,$24,$F3, $06
	.db %00101000,$8C,$F3, $06
	
	.db %00101000,$F8,$F3, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$5C,$F2, $06
	.db %00101000,$8C,$F3, $06
	
	.db %00101000,$A5,$F2, $0C
	.db %00101000,$53,$F5, $0C
	.db %00101000,$53,$F5, $0C
	.db %00101000,$53,$F5, $0C
	
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$24,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$CC,$F2, $06
	.db %00101000,$24,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$33,$F4, $06
	
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$33,$F4, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$F8,$F3, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$53,$F5, $06
	
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$C4,$F1, $06
	
	.db %00101000,$8C,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$24,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$CC,$F2, $06
	.db %00101000,$24,$F3, $06
	.db %00101000,$8C,$F3, $06
	.db %00101000,$33,$F4, $06
	
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$53,$F5, $06
	.db %00101000,$A5,$F2, $06
	
	.db $FF,$00, $00
	.db %10000001,$BF,$F4, $0C
	.db %10000001,$F8,$F3, $06
	.db %10000001,$BF,$F4, $06
	.db %10000001,$F8,$F3, $06
	.db %10000001,$BF,$F4, $06
	.db %10000001,$F8,$F3, $06
	.db %10000001,$55,$F3, $12
	.db %10000001,$F8,$F3, $06
	.db %10000001,$55,$F3, $06
	.db %10000001,$F8,$F3, $06
	.db %10000001,$55,$F3, $06
	.db %10000001,$F8,$F3, $06
	.db %10000001,$55,$F3, $06
	.db $FF,1,55, $00
	
	.db %10000001,$BF,$F4, $0C
	.db %10000001,$F8,$F3, $06
	.db %10000001,$BF,$F4, $06
	.db %10000001,$F8,$F3, $06
	.db %10000001,$BF,$F4, $06
	.db %10000001,$F8,$F3, $06
	.db %10000001,$55,$F3, $12
	.db %10000001,$F8,$F3, $06
	.db %10000001,$55,$F3, $06
	.db %10000001,$8C,$F3, $06
	.db %10000001,$55,$F3, $06
	.db %10000001,$8C,$F3, $06
	.db %10000001,$55,$F3, $06
	
	.db $FF,$FF

;				----==== CHANNEL 1 ====----
	.org $B000
ch1Data:
    .db $00
	;intro
	.db $FF,$00, $00
	.db %00010110,$00,$C4,$41, $0C
	.db %00010110,$00,$2D,$41, $12
	.db %00010110,$00,$0D,$41, $12
	.db %00010110,$00,$FD,$40, $06
	.db %00010110,$00,$E1,$40, $06
	.db %00010110,$00,$C9,$40, $06
	.db %00010110,$00,$FD,$40, $12
	.db %00010110,$00,$0D,$41, $06
	.db %00010110,$00,$53,$41, $06
	.db $FF,3,48, $00
	
	; verse 1
	.db $FF,$00, $00
	.db %11010111,$00,$A9,$40, $0C
	.db %11010111,$00,$53,$41, $06
	.db %11010111,$00,$A9,$40, $0C
	.db %11010111,$00,$C9,$20, $0C
	.db %11010111,$00,$C9,$40, $0C
	.db %11010111,$00,$E1,$40, $0C
	.db %11010111,$00,$0D,$41, $12
	.db %11010111,$00,$53,$41, $06
	.db %11010111,$00,$C4,$41, $06
	
	.db %11010111,$00,$C9,$40, $18
	.db %11010111,$00,$C9,$40, $06
	.db %11010111,$00,$E1,$40, $0C
	.db %11010111,$00,$FD,$40, $06
	.db %11010111,$00,$0D,$41, $06
	.db %11010111,$00,$FD,$40, $06
	.db %11010111,$00,$E1,$40, $06
	.db %11010111,$00,$0D,$41, $12
	.db %11010111,$00,$2D,$41, $06
	.db %11010111,$00,$93,$41, $06
	
	.db %11010111,$00,$C9,$40, $18
	.db %11010111,$00,$C9,$40, $06
	.db %11010111,$00,$E1,$40, $0C
	.db %11010111,$00,$FD,$40, $06
	.db %11010111,$00,$0D,$41, $06
	.db %11010111,$00,$FD,$40, $06
	.db %11010111,$00,$E1,$40, $06
	.db %11010111,$00,$FD,$40, $12
	.db %11010111,$00,$0D,$41, $06
	.db %11010111,$00,$53,$41, $06
	
	.db %00010111,$00,$C4,$41, $0C
	.db %00010111,$00,$2D,$41, $12
	.db %00010111,$00,$0D,$41, $12
	.db %00010111,$00,$FD,$40, $06
	.db %00010111,$00,$E1,$40, $06
	.db %00010111,$00,$C9,$40, $06
	.db %00010111,$00,$FD,$40, $12
	.db %00010111,$00,$0D,$41, $06
	.db %00010111,$00,$53,$41, $06
	; and here I ~can~ repeat
	.db $FF,1,193, $00
	
	; refrain
	.db %00010110,$00,$A9,$40, $06
	.db %00010110,$00,$C9,$40, $0C
	.db %00010110,$00,$86,$40, $0C
	.db %00010110,$00,$A9,$40, $12
	.db %00010110,$00,$B3,$40, $06
	.db %00010110,$00,$E1,$40, $0C
	.db %00010110,$00,$86,$40, $0C
	.db %00010110,$00,$B3,$40, $12
	
	.db %00010110,$00,$FD,$08, $30
	.db %00010110,$00,$B3,$40, $06
	.db %00010110,$00,$E1,$40, $0C
	.db %00010110,$00,$86,$40, $0C
	.db %00010110,$00,$B3,$40, $12
	
	.db %00010110,$00,$A9,$40, $06
	.db %00010110,$00,$C9,$40, $0C
	.db %00010110,$00,$86,$40, $0C
	.db %00010110,$00,$A9,$40, $12
	.db %00010110,$00,$B3,$40, $06
	.db %00010110,$00,$E1,$40, $0C
	.db %00010110,$00,$86,$40, $0C
	.db %00010110,$00,$B3,$40, $12
	
	.db %00010110,$00,$FD,$40, $0C
	.db %00010110,$00,$C9,$40, $06
	.db %00010110,$00,$96,$40, $0C
	.db %00010110,$00,$A9,$40, $0C
	.db %00010110,$00,$B3,$40, $0C
	.db %00010110,$00,$E1,$40, $0C
	.db %00010110,$00,$A9,$40, $0C
	.db %00010110,$00,$96,$40, $12
	
	; chorus
	.db $FF,$00, $00
	.db %11000011,$00,$53,$41, $06
	.db %11000011,$00,$C4,$41, $06
	.db %11000011,$00,$53,$41, $06
	.db %11000011,$00,$C4,$41, $06
	.db %11000011,$00,$67,$41, $06
	.db %11000011,$00,$C4,$41, $06
	.db %11000011,$00,$67,$41, $06
	.db %11000011,$00,$C4,$41, $06
	.db $FF,8,43, $00
	.db %11000010,$00,$53,$41, $06
	.db %11000010,$00,$C4,$41, $06
	.db %00010110,$00,$E1,$40, $0C
	.db %00010110,$00,$FD,$40, $06
	.db %00010110,$00,$0D,$41, $06
	.db %00010110,$00,$FD,$40, $06
	.db %00010110,$00,$E1,$40, $06
	
	.db %00010110,$00,$C9,$40, $12
	.db %00010110,$00,$A9,$40, $12
	.db %00010110,$00,$C9,$40, $0C
	.db %00010110,$00,$B3,$40, $0C
	.db %00010110,$00,$A9,$40, $06
	.db %00010110,$00,$96,$40, $1E
	
	.db %00010111,$00,$C9,$40, $0C
	.db %00010111,$00,$FD,$40, $06
	.db %00010111,$00,$C9,$40, $0C
	.db %00010111,$00,$B3,$40, $0C
	.db %00010111,$00,$A9,$40, $0C
	.db %00010111,$00,$B3,$40, $06
	.db %00010111,$00,$C9,$40, $06
	.db %00010111,$00,$B3,$40, $06
	.db %00010111,$00,$C9,$40, $06
	.db %00010111,$00,$B3,$40, $06
	.db %00010111,$00,$C9,$40, $06
	.db %00010111,$00,$E1,$40, $06
	
	.db %00010111,$00,$FD,$40, $06
	.db %00010111,$00,$E1,$40, $06
	.db %00010111,$00,$C9,$40, $06
	.db %00010111,$00,$FD,$40, $06
	.db %00010111,$00,$E1,$40, $06
	.db %00010111,$00,$C9,$40, $06
	.db %00010111,$00,$B3,$40, $06
	.db %00010111,$00,$E1,$40, $06
	.db %00010111,$00,$A9,$40, $0C
	.db %11001100,$00,$E1,$40, $18
	.db %11001100,$00,$C9,$40, $0C
	
	.db %11001100,$00,$E1,$40, $12
	.db %11001100,$00,$B3,$40, $12
	.db %11001100,$00,$E1,$40, $0C
	.db %11001100,$00,$B3,$40, $12
	.db %11001100,$00,$E1,$40, $12
	.db %11001100,$00,$B3,$40, $0C
	
	.db %11001100,$00,$E1,$40, $12
	.db %11001100,$00,$A9,$40, $12
	.db %11001100,$00,$E1,$40, $0C
	.db %11001100,$00,$A9,$40, $12
	.db %11001100,$00,$E1,$40, $12
	.db %11001100,$00,$A9,$40, $0C
	
	.db %11001100,$00,$E1,$40, $12
	.db %11001100,$00,$B3,$40, $12
	.db %11001100,$00,$E1,$40, $0C
	.db %11001100,$00,$B3,$40, $12
	.db %11001100,$00,$E1,$40, $12
	.db %11001100,$00,$B3,$40, $0C
	
	.db %10011000,$00,$A9,$40, $18
	.db %10011000,$00,$A9,$40, $06
	.db %10011000,$00,$C9,$40, $0C
	.db %10011000,$00,$A9,$40, $0C
	.db %10011000,$00,$C9,$40, $06
	.db %10011000,$00,$E1,$40, $18
	.db %00010100,$00,$0D,$41, $06
	.db %00010100,$00,$FD,$40, $06
	
	.db $FF,$00, $00
	.db %00010100,$00,$C9,$40, $0C
	.db %00010100,$00,$FD,$40, $0C
	.db %00010100,$00,$FD,$40, $06
	.db %00010100,$00,$E1,$40, $06
	.db %00010100,$00,$FD,$40, $06
	.db %00010100,$00,$B3,$40, $0C
	.db %00010100,$00,$FD,$40, $06
	.db %00010100,$00,$A9,$40, $06
	.db %00010100,$00,$B3,$40, $06
	.db %00010100,$00,$A9,$40, $06
	.db %00010100,$00,$B3,$40, $06
	.db %00010100,$00,$E1,$40, $06
	.db %00010100,$00,$FD,$40, $06
	.db $FF,1,68, $00
	
	.db %00010100,$00,$C9,$08, $30
	.db %00010100,$00,$E1,$40, $18
	.db %00010100,$00,$FD,$40, $18
	.db $FF,$FF

;				----==== CHANNEL 3 ====----	
	.org $BA00
ch3Data:
    .db $00
	; intro
	.db $FF,$00, $00
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,3,71, $00
	
	; verse
	.db $FF,$00, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,6,19, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,$00, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,6,19, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $FF,$00, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,6,19, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,$00, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,5,19, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $03 ; - bd
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $03 ; - bd
	.db $01,$0F,$D6, $03 ; - bd
	.db $02,$09,$32, $03 ; - snare 2*
	.db $02,$09,$32, $03 ; - snare 2*
	.db $02,$09,$32, $03 ; - snare 2*
	.db $02,$09,$32, $03 ; - snare 2*
	
	; refrain
	.db $FF,$00, $00
	.db $02,$09,$32, $0C ; - snare 2+
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $0C ; - bd
	.db $01,$0F,$D6, $0C ; - bd
	.db $FF,6,23, $00
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $02,$09,$32, $06 ; - snare 2
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	
	; chorus
	.db $FF,$00, $00
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$01,$0E, $06 ; - hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,8,35, $00
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $FF,$00, $00
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,2,35, $00
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $04,$03,$0E, $06 ; - splash hat
	.db $02,$09,$32, $03 ; - snare 2*
	.db $02,$09,$32, $03 ; - snare 2*
	.db $02,$09,$32, $03 ; - snare 2*
	.db $02,$09,$32, $03 ; - snare 2*
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	
	.db $04,$03,$0E, $0C ; - splash hat
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $FF,$00, $00
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $FF,5,35, $00
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $02,$09,$32, $03 ; - snare 2*
	.db $02,$09,$32, $03 ; - snare 2*
	
	.db $04,$03,$0E, $0C ; - splash hat
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $04,$03,$0E, $0C ; - splash hat
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $02,$09,$32, $06 ; - snare 2
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $02,$09,$32, $06 ; - snare 2
	
	.db $04,$03,$0E, $0C ; - splash hat
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $01,$0F,$D6, $06 ; - bd
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$0F,$D6, $06 ; - bd
	
	.db $04,$03,$0E, $0C ; - splash hat
	.db $02,$09,$32, $06 ; - snare 2
	.db $02,$09,$32, $06 ; - snare 2
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $01,$08,$32, $03 ; - snare
	.db $02,$09,$32, $02 ; - snare 2
	.db $02,$09,$32, $02 ; - snare 2
	.db $02,$09,$32, $02 ; - snare 2
	.db $02,$09,$32, $02 ; - snare 2
	.db $02,$09,$32, $02 ; - snare 2
	.db $02,$09,$32, $02 ; - snare 2
	.db $FF,$FF
	
	.db $FF,$00, $06, $FF,254,6, $00 ; pause indefinitely
	.db $16,$03,$4E, $06 ; - hat 2
	.db $01,$01,$0E, $06 ; - hat
	.db $04,$03,$0E, $06 ; - splash hat
	.db $01,$08,$32, $03 ; - snare
	.db $02,$09,$32, $06 ; - snare 2
	
	.bank 2

	.org $C000

	.bank 3
	.org $E000


; generic footer
	.org $fffa
	.dw nmi		;//NMI
	.dw start	;//Reset
	.dw irq		;//IRQ

; mandatory chr data
	.bank 4
	.org $0000
	.incbin "fullascii.chr"
	.incbin "quarters.chr"
	